//::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
//
//        This file is part of E-Cell Simulation Environment package
//
//                Copyright (C) 1996-2002 Keio University
//
//::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
//
//
// E-Cell is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
// 
// E-Cell is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public
// License along with E-Cell -- see the file COPYING.
// If not, write to the Free Software Foundation, Inc.,
// 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
// 
//END_HEADER
//
// written by Koichi Takahashi <shafi@e-cell.org>,
// E-Cell Project.
//

#ifndef __LIBECS_HPP
#define __LIBECS_HPP

#include "Defs.hpp"


namespace libecs
{

  /** @defgroup libecs The Libecs library
   * The libecs library
   * @{ 
   */ 
  
  /** @file */


  const int MAJOR_VERSION( ECELL_MAJOR_VERSION );
  const int MINOR_VERSION( ECELL_MINOR_VERSION );
  const int MICRO_VERSION( ECELL_MICRO_VERSION );

  const char* const VERSION_STRING( ECELL_VERSION_STRING );


  inline const int getMajorVersion()
  {
    return MAJOR_VERSION;
  }

  inline const int getMinorVersion()
  {
    return MINOR_VERSION;
  }

  inline const int getMicroVersion()
  {
    return MICRO_VERSION;
  }

  inline const std::string getVersion()
  {
    return VERSION_STRING;
  }


  void setDMSearchPath( const String& path );
  const String getDMSearchPath();
  const String getLoadedDMList();

  //  DECLARE_CLASS( Polymorph );
  //  const Polymorph getDMInfoList( const String& aDMName );
  //  const String getDMInfo( const String& aDMName,
  //			  const String& aFieldName );

  


  // Forward declarations.


  // string STL containers.
  DECLARE_LIST  ( String, StringList );
  DECLARE_VECTOR( String, StringVector );
  DECLARE_MAP( const String, String, std::less<const String>, StringMap );
  DECLARE_SHAREDPTR( StringList );
  DECLARE_SHAREDPTR( StringVector );


  // classes

  DECLARE_CLASS( System );
  DECLARE_CLASS( Entity );
  DECLARE_CLASS( EntityType );
  DECLARE_CLASS( SystemPath );
  DECLARE_CLASS( FullID );
  DECLARE_CLASS( FullPN );
  DECLARE_CLASS( VariableReference );
  DECLARE_CLASS( Process );
  DECLARE_CLASS( DiscreteEventProcess );
  DECLARE_CLASS( ProcessMaker );
  DECLARE_CLASS( Stepper );
  DECLARE_CLASS( SystemStepper );
  DECLARE_CLASS( Interpolant );
  DECLARE_CLASS( Model );
  DECLARE_CLASS( Scheduler );
  DECLARE_CLASS( StepperEvent );
  DECLARE_CLASS( StepperMaker );
  DECLARE_CLASS( Variable );
  DECLARE_CLASS( VariableMaker );
  DECLARE_CLASS( System );
  DECLARE_CLASS( SystemMaker );
  DECLARE_CLASS( PropertySlotBase );
  //  DECLARE_CLASS( PropertyInterface );
  DECLARE_CLASS( PropertiedClass );
  DECLARE_CLASS( PropertySlotProxy );
  DECLARE_CLASS( Polymorph );
  DECLARE_CLASS( LoggerBroker );
  DECLARE_CLASS( Logger );
  DECLARE_CLASS( LoggerAdapter );
  DECLARE_CLASS( DataPoint );
  DECLARE_CLASS( LongDataPoint );
  DECLARE_CLASS( DataPointAggregator );
  DECLARE_CLASS( DataPointVector );


  // containers
  DECLARE_VECTOR( Polymorph,    PolymorphVector );
  DECLARE_VECTOR( VariablePtr,  VariableVector );
  DECLARE_VECTOR( ProcessPtr,   ProcessVector );
  DECLARE_VECTOR( SystemPtr,    SystemVector );
  DECLARE_VECTOR( StepperPtr,   StepperVector );
  DECLARE_VECTOR( LoggerPtr,    LoggerVector );
  //  DECLARE_VECTOR( PropertySlotPtr, PropertySlotVector );

  // exceptions

  DECLARE_CLASS( Exception );
  DECLARE_CLASS( UnexpectedError );
  DECLARE_CLASS( NotFound );
  DECLARE_CLASS( CantOpen );
  DECLARE_CLASS( BadID );
  DECLARE_CLASS( CallbackFailed );
  DECLARE_CLASS( NoMethod );
  DECLARE_CLASS( NoSlot );
  DECLARE_CLASS( InvalidEntityType );

  DECLARE_MAP ( const String, Polymorph, std::less<const String>,
				PolymorphMap);


  // other reference counted pointer types

  DECLARE_SHAREDPTR( PolymorphVector );
  DECLARE_SHAREDPTR( DataPointVector );
  
  /** @} */ 

} // namespace libecs

#endif // __LIBECS_HPP


/*
  Do not modify
  $Author: shafi $
  $Revision: 2529 $
  $Date: 2005-11-19 10:36:40 +0100 (Sat, 19 Nov 2005) $
  $Locker$
*/
